"""
dolbyio_rest_apis.media.platform
~~~~~~~~~~~~~~~

This module contains the functions to work with the Platform APIs.
"""

from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.access_token import AccessToken

async def get_api_token(
        app_key: str,
        app_secret: str,
        expires_in: int=None,
    ) -> AccessToken:
    r"""
    To make any API call, you must acquire a JWT (JSON Web Token) format API token.
    Make sure to use this API against https://api.dolby.io/v1.

    Note: Even though the OAuth terminology is used in the following APIs, they are not OAuth compliant.

    See: https://docs.dolby.io/media-apis/reference/get-api-token

    Args:
        app_key: Your Dolby.io App Key.
        app_secret: Your Dolby.io App Secret.
        expires_in: (Optional) API token expiration time in seconds.
            The maximum value is 2,592,000, indicating 30 days.
            If no value is specified, the default is 1800, indicating 30 minutes.

    Returns:
        An :class:`AccessToken` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    data = {
        'grant_type': 'client_credentials',
    }
    add_if_not_none(data, 'expires_in', expires_in)

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post_basic_auth(
            app_key=app_key,
            app_secret=app_secret,
            url='https://api.dolby.io/v1/auth/token',
            data=data
        )

    return AccessToken(json_response)



"""
dolbyio_rest_apis.communications.internal.urls
~~~~~~~~~~~~~~~

This module contains the URLs to use to call the REST APIs.
"""

API_URL = 'https://api.voxeet.com'
COMMS_URL = 'https://comms.api.dolby.io'
SESSION_URL = 'https://session.voxeet.com/v1'

def get_api_url() -> str:
    return f'{API_URL}/v1'

def get_monitor_url() -> str:
    return f'{COMMS_URL}/v1/monitor'

def get_comms_url_v2() -> str:
    return f'{COMMS_URL}/v2'

def get_session_url() -> str:
    return SESSION_URL



"""
dolbyio_rest_apis.communications.streaming
~~~~~~~~~~~~~~~

This module contains the functions to work with the streaming API.
"""

from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.communications.internal.urls import get_comms_url_v2
from typing import List

async def start_rtmp(
        access_token: str,
        conference_id: str,
        rtmp_urls: List[str],
    ) -> None:
    r"""
    Starts the RTMP live stream for the specified conference. Once the Dolby.io Communications APIs service starts
    streaming to the target url, a `Stream.Rtmp.InProgress` Webhook event will be sent.
    You must use this API if the conference is protected using enhanced conference access control.

    See: https://docs.dolby.io/communications-apis/reference/start-rtmp

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        rtmp_urls: List of the RTMP endpoints where to send the RTMP stream to.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    payload = {
        'uri': '|'.join(rtmp_urls),
    }

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/rtmp/start',
            payload=payload
        )

async def stop_rtmp(
        access_token: str,
        conference_id: str,
    ) -> None:
    r"""
    Stops an RTMP stream.
    You must use this API if the conference is protected using enhanced conference access control.

    See: https://docs.dolby.io/communications-apis/reference/stop-rtmp

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/rtmp/stop'
        )

async def start_lls(
        access_token: str,
        conference_id: str,
        stream_name: str,
        publishing_token: str,
    ) -> None:
    r"""
    Starts a Low Latency Stream to Millicast.

    See: https://docs.dolby.io/communications-apis/reference/start-rtmp

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        stream_name: The Millicast stream name to which the conference will broadcasted.
        publishing_token:The Millicast publishing token used to identify the broadcaster.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    payload = {
        'stream_name': stream_name,
        'publishingToken': publishing_token,
    }

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/lls/start',
            payload=payload,
        )

async def stop_lls(
        access_token: str,
        conference_id: str,
    ) -> None:
    r"""
    Stops an existing Low Latency Stream to Millicast.

    See: https://docs.dolby.io/communications-apis/reference/stop-lls

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/lls/stop'
        )
